@@ -16,21 +16,19 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <glib/gi18n.h>
-
 #include "gom-adapter.h"
 #include "gom-command.h"
 #include "gom-command-builder.h"
 #include "gom-filter.h"
 #include "gom-resource.h"
 #include "gom-resource-priv.h"
-
-G_DEFINE_TYPE(GomCommandBuilder, gom_command_builder, G_TYPE_OBJECT)
+#include "gom-sorting.h"
 
 struct _GomCommandBuilderPrivate
 {
    GomAdapter *adapter;
    GomFilter *filter;
+   GomSorting *sorting;
    GType resource_type;
    guint limit;
    guint offset;
@@ -38,11 +36,14 @@ struct _GomCommandBuilderPrivate
    GType m2m_type;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE(GomCommandBuilder, gom_command_builder, G_TYPE_OBJECT)
+
 enum
 {
    PROP_0,
    PROP_ADAPTER,
    PROP_FILTER,
+   PROP_SORTING,
    PROP_LIMIT,
    PROP_M2M_TABLE,
    PROP_M2M_TYPE,
@@ -82,6 +83,8 @@ sql_type_for_column (GParamSpec *pspec)
       if (parent_type == G_TYPE_ENUM ||
           parent_type == G_TYPE_FLAGS)
         return "INTEGER";
+      if (pspec->value_type == G_TYPE_BYTES)
+        return "BLOB";
       if (g_param_spec_get_qdata(pspec, GOM_RESOURCE_FROM_BYTES_FUNC) != NULL)
         return "BLOB";
 
@@ -100,6 +103,10 @@ is_mapped (GParamSpec *pspec)
    if (!ret)
      return FALSE;
 
+   /* Ignore read-only properties as we can never write them back */
+   if ((pspec->flags & G_PARAM_WRITABLE) == 0)
+     return FALSE;
+
    ret = (sql_type_for_column(pspec) != NULL);
    if (!ret) {
      g_debug("Property %s not mapped because type is unsupported (%s)",
@@ -122,6 +129,17 @@ is_new_in_version (GParamSpec *pspec,
    return (GPOINTER_TO_INT(g_param_spec_get_qdata(pspec, GOM_RESOURCE_NEW_IN_VERSION)) + 1) == version;
 }
 
+static gboolean
+table_is_new_in_version (GomResourceClass *klass,
+                         guint             version)
+{
+   GParamSpec *primary_pspec;
+
+   primary_pspec = g_object_class_find_property(G_OBJECT_CLASS(klass),
+                                                klass->primary_key);
+   return is_new_in_version(primary_pspec, version);
+}
+
 static void
 add_fields (GString          *str,
             GomResourceClass *klass)
@@ -349,6 +367,32 @@ add_where (GString     *str,
    }
 }
 
+static void
+add_order_by (GString *str,
+              GType        m2m_type,
+              const gchar *m2m_table,
+              GomSorting *sorting)
+{
+   GHashTable *table_map = NULL;
+   gchar *sql;
+
+   if (sorting) {
+      if (m2m_type) {
+         table_map = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                           g_free, g_free);
+         build_map(table_map, m2m_type, m2m_table);
+      }
+
+      sql = gom_sorting_get_sql(sorting, table_map);
+      g_string_append_printf(str, " ORDER BY %s ", sql);
+      g_free(sql);
+
+      if (table_map) {
+         g_hash_table_destroy(table_map);
+      }
+   }
+}
+
 static void
 add_limit (GString *str,
            guint    limit)
@@ -419,8 +463,10 @@ gom_command_builder_build_create (GomCommandBuilder *builder,
 
    pspecs = g_object_class_list_properties(G_OBJECT_CLASS(klass), &n_pspecs);
 
-   /* Create the table if it doesn't already exist*/
-   if (version == 1) {
+   /* Create the table if it doesn't already exist */
+   if (table_is_new_in_version(klass, version)) {
+      g_debug ("Creating table '%s' new in version %d (if it does not exist)", klass->table, version);
+
       str = g_string_new("CREATE TABLE IF NOT EXISTS ");
       add_table_name(str, klass);
       g_string_append(str, "(");
@@ -455,6 +501,9 @@ gom_command_builder_build_create (GomCommandBuilder *builder,
      if (pspecs[i] != primary_pspec &&
          is_mapped(pspecs[i]) &&
          is_new_in_version(pspecs[i], version)) {
+       g_debug ("Adding column '%s' to table '%s' new in version %d",
+                pspecs[i]->name, klass->table, version);
+
        str = g_string_new("ALTER TABLE ");
        add_table_name(str, klass);
        g_string_append(str, " ADD COLUMN ");
@@ -511,6 +560,7 @@ gom_command_builder_build_select (GomCommandBuilder *builder)
    add_joins(str, klass);
    add_m2m(str, klass, priv->m2m_table, priv->m2m_type);
    add_where(str, priv->m2m_type, priv->m2m_table, priv->filter);
+   add_order_by(str, priv->m2m_type, priv->m2m_table, priv->sorting);
    add_limit(str, priv->limit);
    add_offset(str, priv->offset);
 
@@ -759,9 +809,7 @@ gom_command_builder_build_insert (GomCommandBuilder *builder,
    }
 
    g_type_class_unref(klass);
-   if (str) {
-      g_string_free(str, TRUE);
-   }
+   g_string_free(str, TRUE);
    g_free(pspecs);
 
    return command;
@@ -845,11 +893,7 @@ gom_command_builder_build_update (GomCommandBuilder *builder,
    }
 
    g_type_class_unref(klass);
-
-   if (str) {
-      g_string_free(str, TRUE);
-   }
-
+   g_string_free(str, TRUE);
    g_free(pspecs);
 
    return command;
@@ -869,6 +913,7 @@ gom_command_builder_finalize (GObject *object)
 
    g_clear_object(&priv->adapter);
    g_clear_object(&priv->filter);
+   g_clear_object(&priv->sorting);
    g_free(priv->m2m_table);
 
    G_OBJECT_CLASS(gom_command_builder_parent_class)->finalize(object);
@@ -898,6 +943,9 @@ gom_command_builder_get_property (GObject    *object,
    case PROP_FILTER:
       g_value_set_object(value, builder->priv->filter);
       break;
+   case PROP_SORTING:
+      g_value_set_object(value, builder->priv->sorting);
+      break;
    case PROP_LIMIT:
       g_value_set_uint(value, builder->priv->limit);
       break;
@@ -945,6 +993,11 @@ gom_command_builder_set_property (GObject      *object,
       builder->priv->filter = g_value_dup_object(value);
       g_object_notify_by_pspec(object, pspec);
       break;
+   case PROP_SORTING:
+      g_clear_object(&builder->priv->sorting);
+      builder->priv->sorting = g_value_dup_object(value);
+      g_object_notify_by_pspec(object, pspec);
+      break;
    case PROP_LIMIT:
       builder->priv->limit = g_value_get_uint(value);
       g_object_notify_by_pspec(object, pspec);
@@ -985,12 +1038,11 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
    object_class->finalize = gom_command_builder_finalize;
    object_class->get_property = gom_command_builder_get_property;
    object_class->set_property = gom_command_builder_set_property;
-   g_type_class_add_private(object_class, sizeof(GomCommandBuilderPrivate));
 
    gParamSpecs[PROP_ADAPTER] =
       g_param_spec_object("adapter",
-                          _("Adapter"),
-                          _("The GomAdapter."),
+                          "Adapter",
+                          "The GomAdapter.",
                           GOM_TYPE_ADAPTER,
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property(object_class, PROP_ADAPTER,
@@ -998,17 +1050,26 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
 
    gParamSpecs[PROP_FILTER] =
       g_param_spec_object("filter",
-                          _("Filter"),
-                          _("The filter for the command."),
+                          "Filter",
+                          "The filter for the command.",
                           GOM_TYPE_FILTER,
                           G_PARAM_READWRITE);
    g_object_class_install_property(object_class, PROP_FILTER,
                                    gParamSpecs[PROP_FILTER]);
 
+   gParamSpecs[PROP_SORTING] =
+      g_param_spec_object("sorting",
+                          "Sorting",
+                          "The sorting for the command.",
+                          GOM_TYPE_SORTING,
+                          G_PARAM_READWRITE);
+   g_object_class_install_property(object_class, PROP_SORTING,
+                                   gParamSpecs[PROP_SORTING]);
+
    gParamSpecs[PROP_LIMIT] =
       g_param_spec_uint("limit",
-                        _("Limit"),
-                        _("The maximum number of results."),
+                        "Limit",
+                        "The maximum number of results.",
                         0,
                         G_MAXUINT,
                         0,
@@ -1018,8 +1079,8 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
 
    gParamSpecs[PROP_M2M_TABLE] =
       g_param_spec_string("m2m-table",
-                          _("Many-to-many table"),
-                          _("The table to use for many-to-many queries."),
+                          "Many-to-many table",
+                          "The table to use for many-to-many queries.",
                           NULL,
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property(object_class, PROP_M2M_TABLE,
@@ -1027,8 +1088,8 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
 
    gParamSpecs[PROP_M2M_TYPE] =
       g_param_spec_gtype("m2m-type",
-                         _("Many-to-many type"),
-                         _("The type for the join within m2m-table."),
+                         "Many-to-many type",
+                         "The type for the join within m2m-table.",
                          GOM_TYPE_RESOURCE,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_property(object_class, PROP_M2M_TYPE,
@@ -1036,8 +1097,8 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
 
    gParamSpecs[PROP_OFFSET] =
       g_param_spec_uint("offset",
-                        _("Offset"),
-                        _("The number of results to skip."),
+                        "Offset",
+                        "The number of results to skip.",
                         0,
                         G_MAXUINT,
                         0,
@@ -1047,8 +1108,8 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
 
    gParamSpecs[PROP_RESOURCE_TYPE] =
       g_param_spec_gtype("resource-type",
-                         _("Resource Type"),
-                         _("The resource type to query for."),
+                         "Resource Type",
+                         "The resource type to query for.",
                          GOM_TYPE_RESOURCE,
                          G_PARAM_READWRITE);
    g_object_class_install_property(object_class, PROP_RESOURCE_TYPE,
@@ -1064,8 +1125,5 @@ gom_command_builder_class_init (GomCommandBuilderClass *klass)
 static void
 gom_command_builder_init (GomCommandBuilder *builder)
 {
-   builder->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE(builder,
-                                  GOM_TYPE_COMMAND_BUILDER,
-                                  GomCommandBuilderPrivate);
+   builder->priv = gom_command_builder_get_instance_private(builder);
 }
